package jp.co.sds_corp.util.pooh.custom;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

public class RubberBand {

	private final static int WAITING  = 1;
	private final static int DRAGGING = 2;
	private final static int RUBB_START = 1;
	private final static int RUBB_STOP  = 2;

	private int   status_;

	private Point start_ = null;
	private Point stain_ = null;
	private Point currt_ = null;
	private boolean isStain_;
	private int flg = RUBB_STOP; 
	private int mask = InputEvent.BUTTON1_MASK;

	private Component frame;
	private RubberBandmouseListener mouse = null;
	private RubberBandmouseMotionListener mouseMotion = null;

	public void start() {
		if (start_ == null) {
			start_ = new Point(0,0);
			stain_ = new Point(0,0);
			currt_ = new Point(0,0);
		}

		start_.move(0,0);
		stain_.move(0,0);
		currt_.move(0,0);

		isStain_ = false;
		status_  = WAITING;

		flg = RUBB_START; 
		frame.addMouseListener(mouse);
		frame.addMouseMotionListener(mouseMotion);
	}

	public void stop()  { 
		frame.removeMouseListener(mouse);
		frame.removeMouseMotionListener(mouseMotion);
		flg = RUBB_STOP; 
	}

	public RubberBand(Component _frame) {
		mouse       = new RubberBandmouseListener();
		mouseMotion = new RubberBandmouseMotionListener();
		frame = _frame;
	}

	private Rectangle calcRectangle(Point p1, Point p2) { 
		int x, y, w, h;
		x = Math.min(p1.x, p2.x);
		y = Math.min(p1.y, p2.y);
		w = Math.abs(p1.x - p2.x);
		h = Math.abs(p1.y - p2.y);

		if (h==0 || w==0) {
			return null;
		} else {
			return new Rectangle(x, y, w, h);
		}
	} 

	public void paint(Graphics g) {
		Rectangle r;
		if (flg == RUBB_STOP) return;
		switch (status_) {
			case WAITING:
				g.setColor(Color.red);
				g.setXORMode(frame.getBackground());
				r = calcRectangle(start_, currt_);
				if (r != null) {
					g.drawRect(r.x, r.y, r.width, r.height);
					stain_.move(currt_.x, currt_.y);
					isStain_ = false;
					((IRubberBand)frame).doRubberBand(r);
				}
				break;

			case DRAGGING:
				g.setColor(Color.red);
				g.setXORMode(frame.getBackground());
				if (isStain_) {
					// backword rectangle delete
					r = calcRectangle(start_, stain_);
					g.drawRect(r.x, r.y, r.width, r.height);
				}
				r = calcRectangle(start_, currt_);
				if (r != null) {
					g.drawRect(r.x, r.y, r.width, r.height);
					stain_.move(currt_.x, currt_.y);
					isStain_ = true;
				}
				break;
		}
	}

	private void paintBackground(Graphics g) {}
	private class RubberBandmouseMotionListener extends MouseMotionAdapter {
		public void mouseDragged(MouseEvent e) {
			if (flg == RUBB_STOP) return;
			if ((e.getModifiers() & mask) != 0) {
				switch (status_) {
					case DRAGGING:
						currt_.move(e.getX(), e.getY());
						paint(frame.getGraphics());
						break;
				}
			}
		}
	}

	private class RubberBandmouseListener extends MouseAdapter {
		public void mousePressed(MouseEvent e) {
			if (flg == RUBB_STOP) return;
			if ((e.getModifiers() & mask) != 0) {
				switch (status_) {
					case WAITING:
						start_.move(e.getX(), e.getY());
						currt_.move(e.getX(), e.getY());
						status_ = DRAGGING;
						break;
				}
			}
		}

		public void mouseReleased(MouseEvent e) {
			if (flg == RUBB_STOP) return;
			if ((e.getModifiers() & mask) != 0) {
				switch (status_) {
					case DRAGGING:
						status_ = WAITING;
						if (!start_.equals(e.getPoint()))
							paint(frame.getGraphics());
						break;
				}
			}
		}
	}
}
