package	jp.co.sds_corp.util.pooh.simple;

import java.awt.BorderLayout;
import java.awt.Font;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JViewport;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import javax.swing.text.PlainDocument;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

import jp.co.sds_corp.util.common.CmnFileWriter;

public class SimpleEditorPane extends JPanel {

	protected UndoableEditListener undoHandler = new UndoHandler();
	protected UndoManager undo = new UndoManager();
	private JTextComponent editor;

	public SimpleEditorPane() {
		super(true);
	
		setBorder(BorderFactory.createEtchedBorder());
		setLayout(new BorderLayout());

		editor = createEditor();
		editor.getDocument().addUndoableEditListener(undoHandler);
		editor.setFont(new Font("monospaced", Font.PLAIN, 12));

		JScrollPane scroller = new JScrollPane();
		JViewport port = scroller.getViewport();
		port.add(editor);
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add("Center", scroller);
		add("Center", panel);
	}
	protected JTextComponent getEditor() { return editor; }
	protected void resetUndoManager() { undo.discardAllEdits(); }
	protected JTextComponent createEditor() {
		JTextComponent c = new JTextArea();
		c.setDragEnabled(true);
		return c;
	}

	public void save(String directory, String file) throws IOException, BadLocationException {
		try {
			CmnFileWriter writer = new CmnFileWriter();
			writer.open(directory + "/" + file);
			PlainDocument doc = (PlainDocument)getEditor().getDocument();
			String text = doc.getText(0, doc.getLength());
			writer.write(text);
			writer.close();
		} catch (IOException io) {
			System.err.println("IOException:" + io.getMessage());
			throw io;
		} catch (BadLocationException bad) {
			System.err.println("BadLocationException:" + bad.getMessage());
			throw bad;
		}
	}

	public void saveAs(File fileName) {
	}

	public void redo() {
		try {
			undo.redo();
		} catch (CannotRedoException ex) {}
	}

	public void undo() {
		try {
			undo.undo();
		} catch (CannotUndoException ex) {}
	}

	public void newAction() {
		Document oldDoc = getEditor().getDocument();
		if (oldDoc != null) {
			oldDoc.removeUndoableEditListener(undoHandler);
			getEditor().setDocument(new PlainDocument());
			((JTextArea)getEditor()).setTabSize(4);
			getEditor().getDocument().addUndoableEditListener(undoHandler);
			resetUndoManager();
			revalidate();
		}
	}

	public void openAction(String directory, String file) {
		File f = new File(directory, file);
		if (f.exists()) {
			Document oldDoc = getEditor().getDocument();

			if (oldDoc != null) {
				oldDoc.removeUndoableEditListener(undoHandler);
			}
			getEditor().setDocument(new PlainDocument());
			((JTextArea)getEditor()).setTabSize(4);
			Thread loader = new FileLoader(f, editor.getDocument());
			loader.start();
		}
	}

	public class UndoHandler implements UndoableEditListener {
		public void undoableEditHappened(UndoableEditEvent e) {
			undo.addEdit(e.getEdit());
		}
	}
	public void copy()  { editor.copy();  }
	public void cut()   { editor.cut();   }
	public void paste() { editor.paste(); }

	public class FileLoader extends Thread {
		public FileLoader(File f, Document doc) {
			setPriority(4);
			this.f = f;
			this.doc = doc;
		}

		public void run() {
			try {
				Reader in = new FileReader(f);
				char [] buff = new char[4096];
				int nch;
				while ((nch = in.read(buff, 0, buff.length)) != -1) {
					doc.insertString(doc.getLength(), new String(buff, 0, nch), null);
				}

				doc.addUndoableEditListener(undoHandler);
				resetUndoManager();

			} catch (IOException e) {
				System.out.println(e.toString());
			} catch (BadLocationException e) {
				System.out.println(e.getMessage());
			}
		}
		Document doc;
		File f;
	}
}
