package	jp.co.sds_corp.util.pooh.simple;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenuItem;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import jp.co.sds_corp.util.pooh.list.IList;

/**
*
*/
public class SimpleListAction implements PopupMenuListener, ActionListener {
	public boolean popup;
	public boolean mouse;
	public boolean key;

	private IList list = null;
	private SimplePopupMenu popupMenu = null;
	private ListMouseListener listMouseListener = null;
	private ListKeyListener listKeyListener = null;

	public SimpleListAction(IList _list, boolean _popup, boolean _key, boolean _mouse) {
        if (_popup) {
            popupMenu = new SimplePopupMenu(this, _list.getPopupMenuItem());
            popupMenu.addPopupMenuListener(this);
        }

		if (_key) {
			listKeyListener = new ListKeyListener(this);
			_list.getListComponent().addKeyListener(listKeyListener);
		}

		if (_mouse) {
			listMouseListener = new ListMouseListener(this);
			_list.getListComponent().addMouseListener(listMouseListener);
		}
		list = _list;
		popup = _popup;
		key   = _key;
		mouse = _mouse;
	}

	public void actionPerformed(ActionEvent e) {
		JMenuItem source = (JMenuItem)e.getSource();
		list.doPopupMenuSelected(source);
	}

    private boolean maybeShowPopup(MouseEvent e) {
        boolean bRet = false;
        int row = list.getListComponent().locationToIndex(e.getPoint());
        popupMenu.setSelectedRow(row);
        if (popupMenu.getSelectedRow() != -1) {
            popupMenu.showpopup(e);
            popupMenu.requestFocus();
            list.getListComponent().setSelectedIndex(popupMenu.getSelectedRow());
            bRet = true;
        }
        return bRet;
    }

	public void listClicked(MouseEvent e) {
		int c_cnt = e.getClickCount();
		int modif = e.getModifiers();

		int row = list.getListComponent().locationToIndex(e.getPoint());
		if (row >= 0) {
			if ((modif&InputEvent.BUTTON1_MASK) != 0) {
				switch (c_cnt) {
					case 1: list.doSingleClicked(); break;
					case 2: list.doDoubleClicked(); break;
				}
			} 
		}
	}

	public void listPressed(KeyEvent e) {
		char ch = e.getKeyChar();
		switch (ch) {
			case '\n':
				list.doPressedCR();
				e.consume();
				return;
		}
	}

	public void listReleased(KeyEvent e) {
		int code = e.getKeyCode();
		switch (code) {
			case KeyEvent.VK_UP:
			case KeyEvent.VK_DOWN:
				list.doPressedUpdown(code);
				return;
		}
	}

	public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
		list.doPopupMenuVisible();
	}

	public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
		list.getListComponent().requestFocus(); 
	}

	public void popupMenuCanceled(PopupMenuEvent e) {
		list.getListComponent().requestFocus();
	}

	private class ListMouseListener extends MouseAdapter {
		private SimpleListAction action = null;
		public ListMouseListener(SimpleListAction _action) { action = _action; }
		public void mouseClicked(MouseEvent e)  { action.listClicked(e); }
		public void mouseReleased(MouseEvent e) { 
			if (popup) {
				action.maybeShowPopup(e); 
			}
		}
	}

	private class ListKeyListener extends KeyAdapter {
		private SimpleListAction action = null;
		public ListKeyListener(SimpleListAction _action) { action = _action; }
		public void keyPressed(KeyEvent e) { action.listPressed(e); }
		public void keyReleased(KeyEvent e) { action.listReleased(e); }
	}
}
