package	jp.co.sds_corp.util.pooh.simple;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenuItem;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import jp.co.sds_corp.util.pooh.sheet.Sheet;

/**
*
*/
public class SimpleSheetAction implements PopupMenuListener, ActionListener {
	private Sheet sheet = null;
	private SimplePopupMenu popupMenu = null;
	private TableMouseListener tableMouseListener = null;
	private TableKeyListener tableKeyListener = null;
	private boolean popup;
	private boolean mouse;
	private boolean key;

	public SimpleSheetAction(Sheet _sheet, boolean _popup, boolean _key, boolean _mouse) {
		if (_popup) {
			popupMenu = new SimplePopupMenu(this, _sheet.getPopupMenuItem());
			popupMenu.addPopupMenuListener(this);
		}

		if (_key) {
			tableKeyListener = new TableKeyListener(this);
			_sheet.getTableComponent().addKeyListener(tableKeyListener);
		}

		if (_mouse) {
			tableMouseListener = new TableMouseListener(this);
			_sheet.getTableComponent().addMouseListener(tableMouseListener);
		}
		sheet = _sheet;
		popup = _popup;
		key   = _key;
		mouse = _mouse;
	}

	public void actionPerformed(ActionEvent e) {
		JMenuItem source = (JMenuItem)e.getSource();
		sheet.doPopupMenuSelected(source);
	}

    private boolean maybeShowPopup(MouseEvent e) {
        boolean bRet = false;
        int row = sheet.getTableComponent().rowAtPoint(e.getPoint());
        popupMenu.setSelectedRow(row);
        if (popupMenu.getSelectedRow() != -1) {
            popupMenu.showpopup(e);
            popupMenu.requestFocus();
            sheet.getTableComponent().changeSelection(popupMenu.getSelectedRow(), -1, false, false);
            bRet = true;
        }
        return bRet;
    }

	public void sheetClicked(MouseEvent e) {
		int c_cnt = e.getClickCount();
		int modif = e.getModifiers();

		if ((modif&InputEvent.BUTTON1_MASK) != 0) {
			switch (c_cnt) {
				case 1: sheet.doSingleClicked(); break;
				case 2: sheet.doDoubleClicked(); break;
			}
		} 
	}

	public void sheetPressed(KeyEvent e) {
		char ch = e.getKeyChar();
		switch (ch) {
			case '\n':
				sheet.doPressedCR();
				e.consume();
				return;
		}
	}

	public void sheetReleased(KeyEvent e) {
		int code = e.getKeyCode();
		switch (code) {
			case KeyEvent.VK_UP:
			case KeyEvent.VK_DOWN:
				sheet.doPressedUpdown(code);
				return;
		}
	}

	public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
		sheet.doPopupMenuVisible();
	}

	public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
		sheet.getTableComponent().requestFocus(); 
	}

	public void popupMenuCanceled(PopupMenuEvent e) {
		sheet.getTableComponent().requestFocus();
	}

	private class TableMouseListener extends MouseAdapter {
		private SimpleSheetAction action = null;
		public TableMouseListener(SimpleSheetAction _action) { action = _action; }
		public void mouseClicked(MouseEvent e) { action.sheetClicked(e); }
		public void mouseReleased(MouseEvent e) { 
			if (popup) {
				action.maybeShowPopup(e); 
			}
		}
	}

	private class TableKeyListener extends KeyAdapter {
		private SimpleSheetAction action = null;
		public TableKeyListener(SimpleSheetAction _action) { action = _action; }
		public void keyPressed(KeyEvent e) { action.sheetPressed(e); }
		public void keyReleased(KeyEvent e) { action.sheetReleased(e); }
	}
}
