package	jp.co.sds_corp.util.prop;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

public class CmnPropParser {
	PropParser parser = new PropParser();
	public CmnPropParser() {}

	public ELM parseFile(String baseName) throws Exception {
		parser.parseFile(baseName);
		return parser.getDocument();
	}

	public ELM[] getSpecificELMsByELM(ELM elm, String name) {
		List list = new ArrayList();
		LinkedList children = elm.getChildren();
		if (children != null) {
			int size = children.size();
			for (int i=0; i<size; i++) {
				ELM child = (ELM)children.get(i);
				if (name.equals(child.getName())) {
					list.add(child);
				}
			}
		}
		ELM [] result = new ELM[list.size()];
		return ((ELM [])list.toArray(result));
	}

	String fullname = "";
	public String getAttributeByELM(ELM elm, String name) {
		fullname = "";
		String key = parser.getName(elm) + elm.getPropName() + "." + name;
		String value = parser.getProperty(key);
		return (value==null) ? "" : value;
	}

	protected class PropParser {
		protected Properties Prop = new Properties();
		protected LinkedList keyList = new LinkedList();
		protected ELM rootELM = null;

		public ELM getDocument() { return rootELM; }
		public PropParser() {}
		public void parseFile(String baseName) throws Exception {
			BufferedReader inReader;	
			ClassLoader loader;
			String path;
			URL url;

			loader = getClass().getClassLoader();
			path   = baseName.replace('.','/');
			url    = loader.getResource(path + ".properties");
			Prop.load(url.openStream());

			inReader = new BufferedReader(new InputStreamReader(url.openStream()));
			while (true) {
				String line = inReader.readLine();
				if (line==null) {
					break;
				}
				int idx = line.indexOf("=");
				if (idx >=0 && line.charAt(0) == '#') {
					keyList.add(line.substring(1, idx));
				}
			}
			rootELM = new ELM(null, (String)keyList.get(0), -1);
			deploy(rootELM);
		}

		public String getName(ELM elm) {
			String prop;
			ELM parent = elm.getParentELM();
			if (parent.parent == null) {
				fullname += parent.getPropName() + ".";
				return fullname;
			} else {
				fullname += getName(parent) + parent.getPropName() + ".";
				return fullname;
			}
		}

		public String getProperty(String key) {
			return Prop.getProperty(key);
		}

		private LinkedList split(String key) {
			String token = "";
			StringTokenizer st = new StringTokenizer(key, ".");
			LinkedList list = new LinkedList();
			while (st.hasMoreTokens()) {
				token = st.nextToken();
				list.add(token);
			}
			return list;
		}

		private void deploy(ELM parent) throws Exception {
			for (int i=1; i<keyList.size(); i++) {
				LinkedList list = split((String)keyList.get(i));
				setChild(list);
			}
		}

		private void setChild(LinkedList child) throws Exception {
			ELM parent = rootELM;
			for (int i=1; i<child.size(); i++) {
				String key = (String)child.get(i);
				int index = isDigit(key);
				if (index > 0) {
					continue;
				} else {
					index = ((i+1) < child.size()) ? isDigit((String)child.get(i+1)) : -1;
					ELM me = checkSame(parent, key, index);
					parent = me;
				}
			}
		}

		private ELM checkSame(ELM parent, String key, int index) throws Exception {
			LinkedList list = parent.getChildren();
			if (list != null) {
				Iterator iterator = list.iterator();
				while (iterator.hasNext()) {
					ELM el = (ELM)iterator.next();
					if (el.index == index && key.equals(el.getName())) {
						return el;
					}
				}
			}
			ELM me = new ELM(parent, key, index);
			parent.setChild(me);
			return me;
		}

		private int isDigit(String s) {
			try {
				return Integer.parseInt(s);
			} catch ( NumberFormatException ex ) {
				return -1;
			}
		}
	}
}
