package	jp.co.sds_corp.visualpooh;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JViewport;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreeSelectionModel;

import jp.co.sds_corp.util.pooh.dialog.DialogManager;
import jp.co.sds_corp.util.pooh.dialog.DialogResource;
import jp.co.sds_corp.util.pooh.frame.FrameManager;
import jp.co.sds_corp.util.pooh.frame.FrameResource;
import jp.co.sds_corp.util.pooh.menu.MenuManager;
import jp.co.sds_corp.util.pooh.menu.MenuResource;
import jp.co.sds_corp.util.pooh.menu.PopupResource;
import jp.co.sds_corp.util.pooh.menu.RootMenuResource;
import jp.co.sds_corp.util.pooh.simple.SimpleTree;
import jp.co.sds_corp.util.pooh.toolbar.ToolbarManager;
import jp.co.sds_corp.util.pooh.toolbar.ToolbarResource;
import jp.co.sds_corp.util.window.SizedIcon;
public class PoohResourceTree extends JPanel {

	private DefaultMutableTreeNode toplevelNode;
	private DefaultMutableTreeNode desktopNode;
	private DefaultMutableTreeNode menuNode;
	private DefaultMutableTreeNode frameNode;
	private DefaultMutableTreeNode dialogNode;
	private DefaultMutableTreeNode popupNode;
	private DefaultMutableTreeNode toolbarNode;
	private DefaultMutableTreeNode iconNode;

	private NodeInfo toplevelFolder = null;
	private NodeInfo desktopFolder = null;
	private NodeInfo frameFolder = null;
	private NodeInfo dialogFolder = null;
	private NodeInfo menuFolder = null;
	private NodeInfo popupFolder = null;
	private NodeInfo toolbarFolder = null;
	private NodeInfo iconFolder = null;

	private SimpleTree treeView = null;

	public PoohResourceTree() {
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createLoweredBevelBorder());
	}
	public String getName() { return "Resource View"; }
	public void createTree(VisualPoohDoc doc) {
		ResourceTreeCellRenderer rend = new ResourceTreeCellRenderer();

		/* Toplevel tree */
		toplevelFolder = new NodeInfo(0, 0, 0, "�꥽����");
		toplevelNode = new DefaultMutableTreeNode(toplevelFolder);

		/* Desktop tree */
		desktopFolder = new NodeInfo(1, 1, 0, "desktop");
		desktopNode = new DefaultMutableTreeNode(desktopFolder);
		toplevelNode.add(desktopNode);

		/* Menu tree */
		menuFolder = new NodeInfo(1, 2, 0, "menu");
		menuNode = new DefaultMutableTreeNode(menuFolder);
		toplevelNode.add(menuNode);

		/* frame tree */
		frameFolder = new NodeInfo(1, 3, 0, "frame");
		frameNode = new DefaultMutableTreeNode(frameFolder);
		toplevelNode.add(frameNode);

		/* dialog tree */
		dialogFolder = new NodeInfo(1, 4, 0, "dialog");
		dialogNode = new DefaultMutableTreeNode(dialogFolder);
		toplevelNode.add(dialogNode);

		/* popup tree */
		popupFolder = new NodeInfo(1, 5, 0, "popup");
		popupNode = new DefaultMutableTreeNode(popupFolder);
		toplevelNode.add(popupNode);

		/* toolbar tree */
		toolbarFolder = new NodeInfo(1, 6, 0, "toolbar");
		toolbarNode = new DefaultMutableTreeNode(toolbarFolder);
		toplevelNode.add(toolbarNode);

		/* icon tree */
		iconFolder = new NodeInfo(1, 7, 0, "icon");
		iconNode = new DefaultMutableTreeNode(iconFolder);
		toplevelNode.add(iconNode);

		treeView = new SimpleTree(toplevelNode);
		treeView.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		treeView.setCellRenderer(rend);

		ResourceTreeModel model = new ResourceTreeModel(toplevelNode);
		treeView.setModel(model);

		createDesktopLeaf(doc);
		createMenuLeaf(doc);
		createFrameLeaf(doc);
		createDialogLeaf(doc);
		createPopupLeaf(doc);
		createToolbarLeaf(doc);
		createIconLeaf(doc);

		JScrollPane scroller = new JScrollPane();
		JViewport port = scroller.getViewport();
		port.add(treeView);
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add("Center", scroller);		
		add("Center", panel);
		treeView.expandAll();
	}

	public void createDesktopLeaf(VisualPoohDoc doc) {
		addLeaf(1, 1, doc.getResourceDir(), doc.getDesktop(), "���ʴ��ܾ���");
		addLeaf(1, 2, doc.getResourceDir(), doc.getDesktop(), "����ɹ��ߥ��֥�������");
		addLeaf(1, 3, doc.getResourceDir(), doc.getDesktop(), "��˥塼�ѹ��롼��");
		addLeaf(1, 4, doc.getResourceDir(), doc.getDesktop(), "��ư������ɥ����֥�������");
		addLeaf(1, 5, doc.getResourceDir(), doc.getDesktop(), "��˥塼�С�");
		addLeaf(1, 6, doc.getResourceDir(), doc.getDesktop(), "�ġ���С�");
		addLeaf(1, 7, doc.getResourceDir(), doc.getDesktop(), "���ơ������С�");
	}

	public void createMenuLeaf(VisualPoohDoc doc) {
		MenuManager mManager = doc.getMenuManager();
		Iterator iterator = mManager.iterator();
		while (iterator.hasNext()) {
			MenuResource mr = (MenuResource)iterator.next();
			addLeaf(2, 0, doc.getResourceDir(), doc.getMenu(), mr);
		}
	}

	public void createFrameLeaf(VisualPoohDoc doc) {
		FrameManager fManager = doc.getFrameManager();
		Iterator iterator = fManager.iterator();
		while (iterator.hasNext()) {
			FrameResource fr = (FrameResource)iterator.next();
			addLeaf(3, 0, doc.getResourceDir(), doc.getFrame(), fr);
		}
	}

	public void createDialogLeaf(VisualPoohDoc doc) {
		DialogManager dManager = doc.getDialogManager();
		Iterator iterator = dManager.iterator();
		while (iterator.hasNext()) {
			DialogResource dr = (DialogResource)iterator.next();
			addLeaf(4, 0, doc.getResourceDir(), doc.getDialog(), dr);
		}
	}

	public void createPopupLeaf(VisualPoohDoc doc) {
		PopupResource pResource = doc.getPopupResource();
		if (pResource != null) {
			Iterator iterator = pResource.iterator();
			if (iterator != null) {
				while (iterator.hasNext()) {
					RootMenuResource rmr = (RootMenuResource)iterator.next();
					addLeaf(5, 0, doc.getResourceDir(), doc.getPopup(), rmr);
				}
			}
		}
	}

	public void createToolbarLeaf(VisualPoohDoc doc) {
		ToolbarManager tManager = doc.getToolbarManager();
		if (tManager != null) {
			Iterator iterator = tManager.iterator();
			if (iterator != null) {
				while (iterator.hasNext()) {
					ToolbarResource tbr = (ToolbarResource)iterator.next();
					addLeaf(6, 0, doc.getResourceDir(), doc.getToolbar(), tbr);
				}
			}
		}
	}

	public void createIconLeaf(VisualPoohDoc doc) {
		Vector iconResource = doc.getIconResources();
		for (int i=0; i<iconResource.size(); i++)
			addLeaf(7, 0, doc.getPoohResourceDir(), null, iconResource.get(i));
	}

	public void removeLeaf(DefaultMutableTreeNode node) {
		((ResourceTreeModel)treeView.getModel()).removeNodeFromParent(node);
	}

	public void addLeaf(int nType, int nLeaf, String _path, String _fileName, Object object) {
		String _name = "";
		String _icon = null;
		String _ppath = null;
		DefaultMutableTreeNode _node = null;
		switch (nType) {
			case 1:
				_node  = desktopNode;
				_name = (String)object;
				_icon = "/resource/desktop.gif";
				break;
			case 2:
				_node = menuNode;
				_name = ((MenuResource)object).getMenubar_name();
				_icon = "/resource/menu.gif";
				break;
			case 3:
				_node = frameNode;
				_name = ((FrameResource)object).getFrame_id();
				_icon = "/resource/frame.gif";
				break;
			case 4:
				_node = dialogNode;
				_name = ((DialogResource)object).getDialog_id();
				_icon = "/resource/dialog.gif";
				break;
			case 5:
				_node = popupNode;
				_name = ((RootMenuResource)object).getName();
				_icon = "/resource/menu.gif";
				break;
			case 6:
				_node = toolbarNode;
				_name = ((ToolbarResource)object).getToolbar_name();
				_icon = "/resource/toolbar.gif";
				break;
			case 7:
				_node = iconNode;
				_name = (String)object;
				_icon = _name;
				_ppath = _path; 
				break;
		}
		NodeInfo info = new NodeInfo(2, nType, nLeaf, _name);
		info.setPath(_path);
		info.setFileName(_fileName);
		if (_icon != null) {
			if (_ppath != null)
				info.setIcon(new File(_ppath, _icon));
			else
				info.setIcon(_icon);
		}
		DefaultMutableTreeNode leaf = new DefaultMutableTreeNode(info);
		_node.add(leaf);
		((ResourceTreeModel)treeView.getModel()).reload();
		treeView.expandAll();
	}

	public class NodeInfo {
		String name;
		int level;
		int type;
		int leaf;
		SizedIcon icon;
		String path     = null;
		String fileName = null;

		public NodeInfo(int _level, int _type, int _leaf, String _name) {
			name = _name;
			level = _level;
			type = _type;
			leaf = _leaf;
		}

		public SizedIcon toIcon() { return icon; }
		public String toString() { return name; }
		public int getLevel() { return level; }
		public int getType() { return type; }
		public int getLeaf() { return leaf; }
		public void setPath(String _path) { path = _path; }
		public void setFileName(String _fileName) { fileName = _fileName; }
		public void setIcon(String _icon) {
			java.net.URL img = getClass().getResource(_icon);
			icon = new SizedIcon(img, 16);
		}

		public void setIcon(File file) {
			try {
				java.net.URL img = file.toURL();
				icon = new SizedIcon(img, 16);
			} catch (Exception e){}
		}
	}

	public class ResourceTreeModel extends DefaultTreeModel {
		public boolean isLeaf(Object obj) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)obj;
			NodeInfo info = (NodeInfo)node.getUserObject();
			return info.getLevel() > 1;
		}

		public void removeNodeFromParent(MutableTreeNode node) {
			System.out.println("remove:" + node);
			super.removeNodeFromParent(node);
		}

		ResourceTreeModel(TreeNode node) {
			super(node);
		}
	}

	public class ResourceTreeCellRenderer extends DefaultTreeCellRenderer {
		public Component getTreeCellRendererComponent(JTree tree,
								Object value,
								boolean selected,
								boolean expanded,
								boolean leaf,
								int row,
								boolean hasFocus) {
			Component c = super.getTreeCellRendererComponent(tree, value, selected, expanded,
											leaf, row, hasFocus);

			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			NodeInfo nodeInfo = (NodeInfo)node.getUserObject();
			setBackgroundNonSelectionColor(tree.getBackground());

			if (nodeInfo.getLevel() < 2)
				return c;
			SizedIcon icon = nodeInfo.toIcon();
			if (icon != null)  {
				setIcon(icon);
				return this;
			}
			return c;
		}
	}

	public JTree getTreeView() { return treeView; }
}
