package	jp.co.sds_corp.visualpooh;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;

import javax.swing.JMenuItem;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import jp.co.sds_corp.util.pooh.menu.SubMenuResource;
import jp.co.sds_corp.util.pooh.popup.PopupAdapter;
import jp.co.sds_corp.util.pooh.simple.SimplePanel;
import jp.co.sds_corp.util.pooh.simple.SimplePopup;

public class PoohWorkSpace extends SimplePanel {
	private JTabbedPane tabbed;
	private PoohFileTree     poohfileTree;
	private PoohResourceTree poohressTree;
	private PoohFileTree.NodeInfo fileNodeInfo;
	private PoohResourceTree.NodeInfo resourceNodeInfo;
	private SimplePopup popup = null;

	public PoohWorkSpace() { super(); }
	public boolean createPane() { 
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(new Insets(5,5,5,5)));

		VisualPoohDoc doc = (VisualPoohDoc)toplevel.getAttribute("VisualPoohDoc"); 
		if (doc.isLoadUp()) {
			poohfileTree = new PoohFileTree();
			poohfileTree.createTree(doc);
			poohressTree = new PoohResourceTree();
			poohressTree.createTree(doc);
	
			tabbed = new JTabbedPane();
			tabbed.setTabPlacement(JTabbedPane.BOTTOM);
			tabbed.add(poohressTree.getName(), poohressTree);
			tabbed.add(poohfileTree.getName(), poohfileTree);
			tabbed.setSelectedIndex(0);
			add(tabbed, "Center");
			poohfileTree.getTreeView().addMouseListener(new MouseAdapter() {
				public void mouseReleased(MouseEvent e) {
					fileNodeInfo = null; 
					if (e.getClickCount() >= 2)
						fileTreeClicked(e);
				}
			});

			poohressTree.getTreeView().addMouseListener(new MouseAdapter() {
				public void mouseReleased(MouseEvent e) {
					fileNodeInfo = null; 
					if (e.getClickCount() >= 2)
						resourceTreeClicked(e);
				}
			});

//---------- POPUP �ν���Ǥ�
			try {
				popup = new SimplePopup(toplevel.getPopupResource("POOHWORK_SPACE"));
			} catch (Exception e) {e.printStackTrace();}
			popup.setActionListener(new PopupAdapter() {
				public Component getComponent() { return poohressTree.getTreeView(); }
				public boolean isShowOK(MouseEvent e) { return showOK(e); }
				public void doPopupMenuSelected(Object object) { popupMenuSelected(object); }
			}); 
		}
//------------------------------------------------------------------------------
		return true;
	}

	private PoohFileTree.NodeInfo getFileNodeFromLocation(int x, int y) {
		TreePath path = poohfileTree.getTreeView().getPathForLocation(x, y);
		if (path == null) {
			return null;
		} else {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
			PoohFileTree.NodeInfo info = (PoohFileTree.NodeInfo)node.getUserObject();
			return info;
		}
	}

	private PoohResourceTree.NodeInfo getResourceNodeFromLocation(int x, int y) {
		TreePath path = poohressTree.getTreeView().getPathForLocation(x, y);
		if (path == null) {
			return null;
		} else {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
			PoohResourceTree.NodeInfo info = (PoohResourceTree.NodeInfo)node.getUserObject();
			return info;
		}
	}

	private void fileTreeClicked(MouseEvent e) {
		PoohFileTree.NodeInfo info = getFileNodeFromLocation(e.getX(), e.getY());
		if (info == null)
			return;

		if (info.getLevel() != 2)
			return;

		final Object [] objects = new Object[2];
		objects[0] = info.getPath();
		objects[1] = info.toString();
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				String md = "doDisplayPad";
				toplevel.doSwingEvent(PoohEventTypes.POOH_FILE_TREE_CLICKED_EVENT, md, objects);
			}
		});
	}

	private void resourceTreeClicked(MouseEvent e) {
		PoohResourceTree.NodeInfo info = getResourceNodeFromLocation(e.getX(), e.getY());
		if (info == null)
			return;
		if (info.getLevel() != 2)
			return;
		swingEvent(info, 1);
	}

	public void closed() {}
	public void doSystemBtnClose(WindowEvent e) {}

	private void swingEvent(PoohResourceTree.NodeInfo info, int action) {
		int ev = -1;
		String key = null;
		String md = null;
		switch (info.getType()) {
			case 1:
				ev = PoohEventTypes.POOH_RESOURCE_TREE_DESKTOP_PRODUCTS + info.getLeaf() - 1;
				md = "doDesktopPropertySet";
				break;

			case 2:
				ev = PoohEventTypes.POOH_RESOURCE_TREE_MENU_TEST + action;
				md = "doPoohTreeMenu";
				if (info.getLevel() != 1)
					key = info.toString();
				break;

			case 3:
				ev = PoohEventTypes.POOH_RESOURCE_TREE_FRAME_TEST + action;
				md = "doPoohTreeFrame";
				if (info.getLevel() != 1)
					key = info.toString();
				break;

			case 4:
				ev = PoohEventTypes.POOH_RESOURCE_TREE_DIALOG_TEST + action;
				md = "doPoohTreeDialog";
				if (info.getLevel() != 1)
					key = info.toString();
				break;

			case 5:
				ev = PoohEventTypes.POOH_RESOURCE_TREE_POPUP_TEST + action;
				md = "doPoohTreePopup";
				if (info.getLevel() != 1)
					key = info.toString();
				break;

			case 6:
				ev = PoohEventTypes.POOH_RESOURCE_TREE_TOOLBAR_TEST + action;
				md = "doPoohTreeToolbar";
				if (info.getLevel() != 1)
					key = info.toString();
				break;
			case 7:
				ev = PoohEventTypes.POOH_RESOURCE_TREE_ICON_TEST + action;
				md = "doPoohTreeIcon";
				break;
		}

		if (ev != -1) {
			final int event = ev;
			final String name = md;
			final Object [] objects = (key != null) ? new Object[2] : new Object[1];
			objects[0] = new Integer(ev);
			if (objects.length == 2) { objects[1] = key; }
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					toplevel.doSwingEvent(event, name, objects);
				}
			});
		}
	}

	public void addLeaf(int nType, int nLeaf, String _path, String _fileName, Object object) {
		poohressTree.addLeaf(nType, nLeaf, _path, _fileName, object);
		poohressTree.getTreeView().expandRow(nType);
	}

	public void removeLeaf() {
		DefaultMutableTreeNode node;
		node = (DefaultMutableTreeNode)poohressTree.getTreeView().getLastSelectedPathComponent();
		poohressTree.removeLeaf(node);
	}

	public boolean showOK(MouseEvent e) {
		PoohResourceTree.NodeInfo info = getResourceNodeFromLocation(e.getX(), e.getY());
		if (info == null)
			return false;

		popup.setPopupActionSource(info);
		TreePath path = poohressTree.getTreeView().getPathForLocation(e.getX(), e.getY());
		poohressTree.getTreeView().setSelectionPath(path);
		switch (info.getType()) {
			case 0:
			case 1: return false;
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
				if (info.getLevel()==1) {
					popup.setEnabled(false, 0);
					popup.setEnabled(false, 1);
					popup.setEnabled(true,  2);
					popup.setEnabled(false, 3);
				} else {
					popup.setEnabled(true,  0);
					popup.setEnabled(true,  1);
					popup.setEnabled(false, 2);
					popup.setEnabled(true,  3);
				}
				break;
			case 7:
				if (info.getLevel()==1) {
					popup.setEnabled(false, 0);
					popup.setEnabled(false, 1);
					popup.setEnabled(true,  2);
					popup.setEnabled(false, 3);
				} else {
					return false;
				}
		}
		return true;
	}

	public void popupMenuSelected(Object object) {
		PoohResourceTree.NodeInfo info = (PoohResourceTree.NodeInfo)popup.getPopupActionSource();
		SubMenuResource smr = (SubMenuResource)object;
		int idx = popup.getComponentIndex((JMenuItem)smr.getThis());
		swingEvent(info, idx);
	}
}
