package	jp.co.sds_corp.visualpooh;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import jp.co.sds_corp.util.common.CmnFileWriter;
import jp.co.sds_corp.util.pooh.dialog.DialogManager;
import jp.co.sds_corp.util.pooh.dialog.DialogResource;
import jp.co.sds_corp.util.pooh.frame.FrameManager;
import jp.co.sds_corp.util.pooh.frame.FrameResource;
import jp.co.sds_corp.util.pooh.menu.MenuManager;
import jp.co.sds_corp.util.pooh.menu.MenuResource;
import jp.co.sds_corp.util.pooh.menu.PopupParser;
import jp.co.sds_corp.util.pooh.menu.PopupResource;
import jp.co.sds_corp.util.pooh.menu.RootMenuResource;
import jp.co.sds_corp.util.pooh.toolbar.ToolbarManager;
import jp.co.sds_corp.util.pooh.toolbar.ToolbarParser;
import jp.co.sds_corp.util.pooh.toolbar.ToolbarResource;
import jp.co.sds_corp.util.pooh.tools.OutputDesktopResource;
import jp.co.sds_corp.util.pooh.tools.OutputDialogResource;
import jp.co.sds_corp.util.pooh.tools.OutputFrameResource;
import jp.co.sds_corp.util.pooh.tools.OutputMenuResource;
import jp.co.sds_corp.util.pooh.tools.OutputPopupResource;
import jp.co.sds_corp.util.pooh.tools.OutputToolbarResource;
import jp.co.sds_corp.util.pooh.top.DesktopParser;
import jp.co.sds_corp.util.pooh.top.DesktopPartsInfo;
import jp.co.sds_corp.util.pooh.top.DesktopResource;
import jp.co.sds_corp.util.pooh.top.Toplevel;
import jp.co.sds_corp.util.xml.CmnXMLParser;
import jp.co.sds_corp.util.xml.CmnXMLWriter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class VisualPoohDoc {
	private Toplevel toplevel;
	private MenuManager mManager;
	private FrameManager fManager;
	private DialogManager dManager;
	private ToolbarManager tManager;

	private DesktopResource desktopResource;
	private MenuResource menuResource;
	private PopupResource popupResource;
	private Vector iconResource = new Vector();

    public static final String SRC_DIR       = "src";
    public static final String META_INF_DIR  = "META-INF";
    public static final String RESOURCE_DIR  = "resource";
    public static final String FRAME_INF_DIR = "FRAME-INF";
    public static final String INSTALL_DIR   = "install";
    public static final String CHAR_SET      = "eucjis";

	private String top_dir;
	private String title;
	private String pack;
	private String package_name;
	private String m_source;
	private String w_source;
	private Integer app_type;
	private boolean prop_use;
	private boolean modified = false;

	// products infomation
	private String version;
	private String vender;
	private String url;
	private String project;

	// status bar
	private boolean bar_use;
	private String  bar_count;
	private boolean load_up = false;

	// menu
	private boolean [] c_menu = null;

	private File step0, step1, step2, step3, step4, step5; 
	private String [] parent_class = { "SimpleSdiFrame", "SimpleMdiFrame", "SimpleDialog" };

	public void setToplevel(Toplevel _toplevel) { 
		toplevel = _toplevel; 
		mManager = new MenuManager(toplevel);
		fManager = new FrameManager(toplevel);
		dManager = new DialogManager(toplevel);
	}

	public void createDirectory(String key) throws Exception {
		m_source 	    = (String)toplevel.getUserAttribute(key, "MAIN_SOURCE");
		w_source 	    = (String)toplevel.getUserAttribute(key, "WINDOW_SOURCE");
		app_type 	    = (Integer)toplevel.getUserAttribute(key, "APP_STYLE");
		package_name    = (String)toplevel.getUserAttribute(key, "PACKAGE");

		project         = (String)toplevel.getUserAttribute(key, "PROJECT");
		version 		= (String)toplevel.getUserAttribute(key, "VERSION");
		vender 		    = "";
		title		    = project;
		url				= "";

		bar_use         = true;
		bar_count       = "1";

		pack            = package_name.replace('.', '/');
		top_dir         = project + "-" + version + "-" + SRC_DIR;

		prop_use		= true;
		String [] function = (String [])toplevel.getUserAttribute(key, "FUNCTION");
		c_menu = new boolean[function.length];
		for (int i=0; i<c_menu.length; i++)
			c_menu[i] = (function[i].equals("true")) ? true : false;

		File p = new File((String)toplevel.getUserAttribute(key, "POSITION"));
		step0  = new File(p, top_dir + "/" + INSTALL_DIR);
		step1  = new File(p, top_dir + "/" + SRC_DIR + "/" + project + "/" + META_INF_DIR);
		step2  = new File(p, top_dir + "/" + SRC_DIR + "/" + project + "/" + RESOURCE_DIR);
		step3  = new File(p, top_dir + "/" + SRC_DIR + "/" + project + "/" + pack);
		step4  = new File(step3, FRAME_INF_DIR);
		step5  = new File(p, top_dir + "/" + SRC_DIR);

		step0.mkdirs(); // install
		step1.mkdirs(); // META-INF
		step2.mkdirs(); // resource
		step3.mkdirs(); // source
		step4.mkdirs(); // FRAME-INF
	}

	public boolean exists(String path, String file) {
		File f = new File(path, file);
		return f.exists();
	}

	public void writeMainSource() throws Exception {
		if (exists(getSourceDir(), getMainSource())) {
			if (!warning(getMainSource())) return;
		}

		CmnFileWriter w1 = new CmnFileWriter();
		w1.open(getSourceDir() + "/" + getMainSource());
		BufferedReader br = openSkeleton("/resource/Skeleton.txt");

		while (true) {
			String line = br.readLine();
			if (line == null) { break; }
			line = line.replaceAll("@PACKAGE_NAME", package_name);
			line = line.replaceAll("@CLASS_NAME"  ,	m_source);
			w1.println(line);
		}

		br.close();
		w1.close();
	}

	public void writeWindowSource() throws Exception {
		if (exists(getSourceDir(), getWindowSource())) {
			if (!warning(getWindowSource())) return;
		}

		CmnFileWriter w1 = new CmnFileWriter();
		w1.open(getSourceDir() + "/" + getWindowSource());
		BufferedReader br = openSkeleton("/resource/SkeletonWindow.txt");

		while (true) {
			String line = br.readLine();
			if (line == null) { break; }
			line = line.replaceAll("@PACKAGE_NAME", package_name);
			line = line.replaceAll("@CLASS_NAME"  , w_source);
			line = line.replaceAll("@APPLICATION_TYPE", parent_class[app_type.intValue()]);
			w1.println(line);
		}
		br.close();
		w1.close();
	}

	public void writeManifest() throws Exception {
		if (exists(getManifestDir(), getManifest())) {
			if (!warning(getManifest())) return;
		}
		CmnFileWriter w1 = new CmnFileWriter();
		w1.open(getManifestDir() + "/" + getManifest());
		BufferedReader br = openSkeleton("/resource/MANIFEST.txt");

		while (true) {
			String line = br.readLine();
			if (line == null) { break; }
			line = line.replaceAll("@JAVA_VERSION", System.getProperty("java.vm.version"));
			line = line.replaceAll("@PACKAGE_NAME", (package_name + "." + m_source));
			line = line.replaceAll("@CLASS_PATH", "");
			line = line.replaceAll("@PACKAGE_DIR", pack);
			line = line.replaceAll("@TITLE", title);
			line = line.replaceAll("@PRODUCT_VERSION", version);
			line = line.replaceAll("@PRODUCT_VENDOR", "\"" + vender + "\"");
			w1.println(line);
		}

		br.close();
		w1.close();
	}

	public void writeBuild() throws Exception {
		if (exists(getBuildXMLDir(), getBuildXML())) {
			if (!warning(getBuildXML())) return;
		}
		CmnFileWriter w1 = new CmnFileWriter();
		w1.open(getBuildXMLDir() + "/" + getBuildXML());
		BufferedReader br = openSkeleton("/resource/build.txt");

		String home_dir = System.getProperty("user.dir").replace('\\', '/');
		String char_set = System.getProperty("file.encoding");
		String package_home = pack.replace('.', '/');
		while (true) {
			String line = br.readLine();
			if (line == null) { break; }
			line = line.replaceAll("@RELEASE_VERSION", (m_source + " " + version));
			line = line.replaceAll("@APPLICATION_NAME", m_source);
			line = line.replaceAll("@FRAME_INF_DIR", package_home);
			line = line.replaceAll("@HOME_DIR", project);
			line = line.replaceAll("@CHAR_SET", char_set);
			line = line.replaceAll("@POOHS_DIR", home_dir);
			w1.println(line);
		}

		br.close();
		w1.close();
	}

	public void writeRunBat() throws Exception {
		if (exists(getShellDir(), getRunWindows())) {
			if (!warning(getRunWindows())) return;
		}
		CmnFileWriter w1 = new CmnFileWriter();
		w1.open(step0.getPath() + "/" + "runWin.bat");
		BufferedReader br = openSkeleton("/resource/runwindows.txt");
		String char_set = System.getProperty("file.encoding");

		while (true) {
			String line = br.readLine();
			if (line == null) { break; }
			line = line.replaceAll("@RUNTIME_CLASS", package_name + "." + m_source);
			line = line.replaceAll("@RUNTIME_JAR", project + ".jar");
			if (prop_use) {
				line = line.replaceAll("@POOH_RESOURCE", "property");
				line = line.replaceAll("@PROPERTY", "");
				line = line.replaceAll("@XML", "rem");
			} else { 
				line = line.replaceAll("@POOH_RESOURCE", "xml");
				line = line.replaceAll("@XML", "");
				line = line.replaceAll("@PROPERTY", "rem");
			}
			line = line.replaceAll("@POOH_PATH", package_name);
			w1.println(line);
		}

		br.close();
		w1.close();
	}

	public void writeRunSh() throws Exception {
		if (exists(getShellDir(), getRunUnix())) {
			if (!warning(getRunUnix())) return;
		}

		CmnFileWriter w1 = new CmnFileWriter();
		w1.open(getShellDir() + "/" + getRunUnix());
		BufferedReader br = openSkeleton("/resource/rununix.txt");
		String char_set = System.getProperty("file.encoding");

		while (true) {
			String line = br.readLine();
			if (line == null) { break; }
			line = line.replaceAll("@RUNTIME_CLASS", package_name + "." + m_source);
			line = line.replaceAll("@RUNTIME_JAR", project + ".jar");
			if (prop_use) {
				line = line.replaceAll("@POOH_RESOURCE", "properties");
				line = line.replaceAll("@PROPERTY", "");
				line = line.replaceAll("@XML", "#");
			} else { 
				line = line.replaceAll("@POOH_RESOURCE", "xml");
				line = line.replaceAll("@XML", "");
				line = line.replaceAll("@PROPERTY", "#");
			}
			line = line.replaceAll("@POOH_PATH", package_name);
			w1.println(line);
		}
		br.close();
		w1.close();
	}

	public void writePoohResource() throws Exception {
		if (exists(getPoohResourceDir(), getPoohResource())) {
			if (!warning(getPoohResource())) return;
		}

		CmnFileWriter w1 = new CmnFileWriter();
		w1.open(getPoohResourceDir() + "/" + getPoohResource());
		BufferedReader br = openSkeleton("/resource/pooh.resource.txt");
		String char_set = System.getProperty("file.encoding");

		while (true) {
			String line = br.readLine();
			if (line == null) { break; }
			if (prop_use) {
				line = line.replaceAll("@RESOURCE",	"properties");
			} else {
				line = line.replaceAll("@RESOURCE",	"xml");
			}
			line = line.replaceAll("@PATH", package_name);
			line = line.replaceAll("@CHARSET", char_set);
			w1.println(line);
		}

		br.close();
		w1.close();
	}

	public void copyIcon() throws Exception {
		String [] icons = {
			"blank_s", "new_s", "open_s", "save_s", "saveas_s", "print_s", "close_s",
			"cut_s", "copy_s", "paste_s", "undo_s", "redo_s", "pilseup_s", "halign_s",
			"valign_s", "win_s","hlp_s"
		};

		int b;
		for (int i=0; i<icons.length; i++) {
			InputStream is = openIcon("/resource/" + icons[i] + ".gif");
			FileOutputStream os 
				= new FileOutputStream(new File(getPoohResourceDir(), icons[i] + ".gif"));
			while ((b = is.read()) != -1) {
				os.write(b);
			}
			os.close();
			iconResource.add(icons[i] + ".gif");
		}
	}

	public void writeDesktop() throws Exception {
		if (exists(getResourceDir(), getDesktop())) {
			if (!warning(getDesktop())) return;
		}

		Element ele;
		CmnXMLWriter writer = new CmnXMLWriter();
		writer.createDocument("desktop");
		ele = writer.createElement("products");
		writer.setAttribute(ele, "version", version);
		writer.setAttribute(ele, "url", url);
		writer.setAttribute(ele, "copyright", vender);
		writer.setAttribute(ele, "product", project);
		writer.getRoot().appendChild(ele);

		ele = writer.createElement("desktop-attri");
		writer.setAttribute(ele, "name", project);
		writer.setAttribute(ele, "object", (package_name + "." + w_source));
		writer.getRoot().appendChild(ele);

		ele = writer.createElement("desktop-parts");
		Element ele_1 = writer.createElement("parts");
		writer.setAttribute(ele_1, "type", "menuBar");
		writer.setAttribute(ele_1, "key", (project.toUpperCase() + "_MENU"));
		ele.appendChild(ele_1);
		if (bar_use) {
			ele_1 = writer.createElement("parts");
			writer.setAttribute(ele_1, "type", "statusBar");
			writer.setAttribute(ele_1, "barcount", bar_count);
			ele.appendChild(ele_1);
		}
		writer.getRoot().appendChild(ele);
		writer.printFile(new File(step4, "desktop.xml"));
	}

	public void writeMenu() throws Exception {
		if (exists(getResourceDir(), getMenu())) {
			if (!warning(getMenu())) return;
		}
		menuResource = 
			(MenuResource)mManager.createMenu("DEFAULT_STRUCT_MENU", "resource.menu", "property");
		menuResource.setMenubar_name((project.toUpperCase() + "_MENU"));
		OutputMenuResource out = new OutputMenuResource();
		out.setMenus(menuResource.getMenubar_name());
		out.setPartResource(menuResource, 0);
		if (c_menu[0])
			out.setPartResource(menuResource, 1);
		if (c_menu[1])
			out.setPartResource(menuResource, 2);
			menuResource.remove(2);
		if (c_menu[2])
			out.setPartResource(menuResource, 3);
		out.writeResource(step4.getPath() + "/" + "menu.xml");
		mManager.removeAll();
	}


	public void writeFrame() throws Exception {
		if (exists(getResourceDir(), getFrame())) {
			if (!warning(getFrame())) return;
		}
		Element ele;
		CmnXMLWriter writer = new CmnXMLWriter();
		writer.createDocument("gui");
		writer.printFile(new File(step4, "frame.xml"));
	}

	public void writeDialog() throws Exception {
		if (exists(getResourceDir(), getDialog())) {
			if (!warning(getDialog())) return;
		}
		Element ele;
		CmnXMLWriter writer = new CmnXMLWriter();
		writer.createDocument("gui");
		writer.printFile(new File(step4, "dialog.xml"));
	}

	public void writePopup() throws Exception {
		if (exists(getResourceDir(), getPopup())) {
			if (!warning(getPopup())) return;
		}
		Element ele;
		CmnXMLWriter writer = new CmnXMLWriter();
		writer.createDocument("gui");
		writer.printFile(new File(step4, "popup.xml"));
	}

	public void writeToolbar() throws Exception {
		if (exists(getResourceDir(), getToolbar())) {
			if (!warning(getToolbar())) return;
		}
		Element ele;
		CmnXMLWriter writer = new CmnXMLWriter();
		writer.createDocument("gui");
		writer.printFile(new File(step4, "toolbar.xml"));
	}

	public void xml2Prop() throws Exception {
		String char_set = System.getProperty("file.encoding");
		if (char_set.equals("MS932")) {
			char_set = "Shift_JIS";
		}
/*
		new CmnXML2Prop(getResourceDir() + "/" + "desktop.xml",
						getResourceDir() + "/" + "desktop.properties",
						char_set
		);

		new CmnXML2Prop(getResourceDir() + "/" + "frame.xml",
						getResourceDir() + "/" + "frame.properties",
						char_set
		);

		new CmnXML2Prop(getResourceDir() + "/" + "dialog.xml",
						getResourceDir() + "/" + "dialog.properties",
						char_set
		);

		new CmnXML2Prop(getResourceDir() + "/" + "menu.xml",
						getResourceDir() + "/" + "menu.properties",
						char_set
		);

		new CmnXML2Prop(getResourceDir() + "/" + "popup.xml",
						getResourceDir() + "/" + "popup.properties",
						char_set
		);

		new CmnXML2Prop(getResourceDir() + "/" + "toolbar.xml",
						getResourceDir() + "/" + "toolbar.properties",
						char_set
		);
*/
	}

	public void setupResource() throws Exception {
		mManager.loadResourceAll(step4.getPath() + "/menu.xml", "xml");
		fManager.loadResourceAll(step4.getPath(), "/frame.xml", "xml");
		dManager.loadResourceAll(step4.getPath() + "/dialog.xml", "xml");

		PopupParser popupParser = new PopupParser(toplevel);
		popupResource = popupParser.loadResourceAll(step4.getPath() + "/popup.xml", "xml");

		ToolbarParser toolbarParser = new ToolbarParser(toplevel);
		tManager = toolbarParser.loadResourceAll(step4.getPath() + "/toolbar.xml", "xml");

		DesktopParser parser  = new DesktopParser(toplevel, toplevel.getResourcePath());
		desktopResource = parser.loadResource(step4.getPath() + "/desktop.xml", "xml");
		load_up = true;
	}

	private BufferedReader openSkeleton(String path) throws Exception {
		URL url = getClass().getResource(path);
        return new BufferedReader(new InputStreamReader(url.openStream(), CHAR_SET));
	}

	private InputStream openIcon(String path) throws Exception {
		URL url = getClass().getResource(path);
        return url.openStream();
	}

	private boolean warning(String name) {
		Object [] options = { "��񤭤���", "��ߤ���" };
        return toplevel.warningDialog(name + " ��¸�ߤ��ޤ�����񤭤��ޤ�����", "��ǧ", options, 1);
	}

	private boolean confirm(String msg) {
		Object [] options = { "OK" };
        return toplevel.warningDialog(msg, "��ǧ", options, 0);
	}

	public String getPoohResourceDir() { return step2.getPath(); }
	public String getResourceDir() { return step4.getPath(); }
	public String getSourceDir()   { return step3.getPath(); }
	public String getShellDir()     { return step0.getPath(); }
	public String getManifestDir() { return step1.getPath(); }
	public String getBuildXMLDir() { return step5.getPath(); }

	public String getPoohResource()   { return "pooh.properties"; }
	public String getMainSource()   { return m_source + ".java"; }
	public String getWindowSource() { return w_source + ".java"; }
	public String getBuildXML()   { return "build.xml"; }
	public String getManifest()   { return "MANIFEST.MF"; }
	public String getRunWindows() { return "runWin.bat"; }
	public String getRunUnix()    { return "runUX.sh"; }
	public String getDesktop()    { return "desktop.xml"; }
	public String getMenu()       { return "menu.xml"; }
	public String getFrame()      { return "frame.xml"; }
	public String getDialog()     { return "dialog.xml"; }
	public String getPopup()     { return "popup.xml"; }
	public String getToolbar()     { return "toolbar.xml"; }

	public MenuManager getMenuManager()     { return mManager; }
	public FrameManager getFrameManager()   { return fManager; }
	public DialogManager getDialogManager() { return dManager; }

	public void loadProject(File file) throws Exception {
		CmnXMLParser parser = new CmnXMLParser();
		Document doc = parser.parseFile(file);
		if (doc == null) {
			throw new Exception("not tab <VirtualPooh>");
		} else {
			Element [] e;
			e = parser.getSpecificElementsByNode(doc.getDocumentElement(), "resource");
			step4 = new File(parser.getAttributeByElement(e[0],"directory")); 
			parser.getAttributeByElement(e[0], "desktop");
			parser.getAttributeByElement(e[0], "menu");
			parser.getAttributeByElement(e[0], "frame");
			parser.getAttributeByElement(e[0], "dialog");
			parser.getAttributeByElement(e[0], "popup");

			e = parser.getSpecificElementsByNode(doc.getDocumentElement(), "source");
			step3 = new File(parser.getAttributeByElement(e[0],"directory")); 
			m_source = parser.getAttributeByElement(e[0],"main"); 
			w_source = parser.getAttributeByElement(e[0],"window"); 

			e = parser.getSpecificElementsByNode(doc.getDocumentElement(), "build");
			step5 = new File(parser.getAttributeByElement(e[0],"directory")); 
			parser.getAttributeByElement(e[0],"buildfile"); 

			e = parser.getSpecificElementsByNode(doc.getDocumentElement(), "manifest");
			step1 = new File(parser.getAttributeByElement(e[0],"directory")); 
			parser.getAttributeByElement(e[0],"manifestfile"); 

			e = parser.getSpecificElementsByNode(doc.getDocumentElement(), "shell");
			step0 = new File(parser.getAttributeByElement(e[0],"directory")); 
			parser.getAttributeByElement(e[0],"windows"); 
			parser.getAttributeByElement(e[0],"unix"); 

			e = parser.getSpecificElementsByNode(doc.getDocumentElement(), "pooh");
			step2 = new File(parser.getAttributeByElement(e[0],"directory")); 
			parser.getAttributeByElement(e[0],"name"); 

			e = parser.getSpecificElementsByNode(doc.getDocumentElement(), "projects");
			project = parser.getAttributeByElement(e[0],"project"); 
			package_name = parser.getAttributeByElement(e[0],"package"); 

			e = parser.getSpecificElementsByNode(doc.getDocumentElement(), "icons");
			Element [] ee = parser.getSpecificElementsByNode(e[0], "icon");
			for (int i=0; i<ee.length; i++) {
				String icon_name = parser.getAttributeByElement(ee[i],"name"); 
				iconResource.add(icon_name);
			}

			load_up = true;
		}
	}

	public void save() {
		Element ele;
		try {
			if (!load_up)  return;
			CmnXMLWriter writer = new CmnXMLWriter();
			writer.createDocument("VisualPooh");

			ele = writer.createElement("resource");
			writer.setAttribute(ele, "directory", getResourceDir());
			writer.setAttribute(ele, "desktop",   getDesktop());
			writer.setAttribute(ele, "menu",      getMenu());
			writer.setAttribute(ele, "frame",     getFrame());
			writer.setAttribute(ele, "dialog",    getDialog());
			writer.setAttribute(ele, "popup",     getPopup());
			writer.setAttribute(ele, "toolbar",   getToolbar());
			writer.getRoot().appendChild(ele);

			ele = writer.createElement("source");
			writer.setAttribute(ele, "directory", getSourceDir());
			writer.setAttribute(ele, "main",      m_source);
			writer.setAttribute(ele, "window",    w_source);
			writer.getRoot().appendChild(ele);

			ele = writer.createElement("build");
			writer.setAttribute(ele, "directory", getBuildXMLDir());
			writer.setAttribute(ele, "buildfile", getBuildXML());
			writer.getRoot().appendChild(ele);

			ele = writer.createElement("manifest");
			writer.setAttribute(ele, "directory",    getManifestDir());
			writer.setAttribute(ele, "manifestfile", getManifest());
			writer.getRoot().appendChild(ele);

			ele = writer.createElement("shell");
			writer.setAttribute(ele, "directory", getShellDir());
			writer.setAttribute(ele, "window",    getRunWindows());
			writer.setAttribute(ele, "unix",      getRunUnix());
			writer.getRoot().appendChild(ele);

			ele = writer.createElement("projects");
			writer.setAttribute(ele, "project", project);
			writer.setAttribute(ele, "package", package_name);
			writer.getRoot().appendChild(ele);

			ele = writer.createElement("pooh");
			writer.setAttribute(ele, "directory", getPoohResourceDir());
			writer.setAttribute(ele, "name", getPoohResource());
			writer.getRoot().appendChild(ele);

			ele = writer.createElement("icons");
			for (int i=0; i<iconResource.size(); i++) {
				Element eee = writer.createElement("icon");
				writer.setAttribute(eee, "name", (String)iconResource.get(i));
				ele.appendChild(eee);
			}

			writer.getRoot().appendChild(ele);
			writer.printFile(new File(step5, (project + ".xml")));

			if (!isModified()) return;
			Iterator iterator;
			OutputFrameResource ofr = new OutputFrameResource();
			iterator = fManager.iterator();
			while (iterator.hasNext())
				ofr.setPartResource((FrameResource)iterator.next());
			ofr.writeResource(step4.getPath() + "/" + "frame.xml");

			OutputDialogResource odr = new OutputDialogResource();
			iterator = dManager.iterator();
			while (iterator.hasNext())
				odr.setPartResource((DialogResource)iterator.next());
			odr.writeResource(step4.getPath() + "/" + "dialog.xml");

			OutputMenuResource omr = new OutputMenuResource();
			iterator = mManager.iterator();
			while (iterator.hasNext()) {
				MenuResource mr = (MenuResource)iterator.next();
				omr.setMenus(mr.getMenubar_name());
				omr.setAllResource(step4.getPath() + "/" + "menu.xml", mr);
			}

			OutputPopupResource opr = new OutputPopupResource();
			opr.setAllResource(step4.getPath() + "/" + "popup.xml", popupResource);

			OutputToolbarResource otr = new OutputToolbarResource();
			otr.setAllResource(step4.getPath() + "/" + "toolbar.xml", tManager);

			OutputDesktopResource osr = new OutputDesktopResource();
			osr.setAllResource(step4.getPath() + "/" + "desktop.xml", desktopResource); 

			xml2Prop();
			setModified(false);
		} catch (Exception e) { e.printStackTrace(); }
	}

	public void close() { 
		save(); load_up = false; 
		mManager.removeAll();
		fManager.removeAll();
		dManager.removeAll();
		iconResource.clear();
	}

	public boolean isLoadUp() { return load_up; }
	public DesktopResource getDesktopResource() { return desktopResource; }
	public FrameResource getFrameResource(String fn) { return (FrameResource)fManager.get(fn); }
	public MenuResource getMenuResource(String fn) { return (MenuResource)mManager.get(fn); }
	public PopupResource getPopupResource() {  return popupResource; }
	public RootMenuResource getPopupRootMenuResource(String fn) { 
		return (RootMenuResource)popupResource.getRootMenuResource(fn); 
	}

	public DialogResource getDialogResource(String fn) { return (DialogResource)dManager.get(fn); }

	public boolean addFrame(FrameResource res) { 
		if (fManager.isAlive(res.getFrame_id())) {
			confirm(res.getFrame_id()+" �ϡ��ɲäǤ��ޤ���");
			return false;
		}
		fManager.add(res.getFrame_id(), res); 
		return true;
	}
	public void removeFrame(String fn) { fManager.remove(fn); }

	public boolean addDialog(DialogResource res) {
		if (dManager.isAlive(res.getDialog_id())) {
			confirm(res.getDialog_id()+" �ϡ��ɲäǤ��ޤ���");
			return false;
		}
		dManager.add(res.getDialog_id(), res);
		return true;
	}
	public void removeDialog(String fn) { dManager.remove(fn); }

	public MenuResource createEmptyMenuResource(String menu) throws Exception {
		MenuResource r;
		if (mManager.isAlive(menu)) {
			confirm(menu+" �ϡ��ɲäǤ��ޤ���");
			r = null;
		} else {
			r = new MenuResource();
			r.setMenubar_name(menu);
			mManager.add(menu, r);
		}
		return r;
	}

	public boolean removeMenu(String fn) { 
		boolean r = true;
		DesktopResource dr = getDesktopResource();
		LinkedList list = dr.getDesktopPartsLists();
		Iterator iterator = list.iterator();
		while (iterator.hasNext()) {
			DesktopPartsInfo info = (DesktopPartsInfo)iterator.next();
			if (info.getInt_type() == DesktopPartsInfo.MENU_BAR) {
				if (info.getKey().equals(fn)) {
					r = false;
					break;
				}	
			}
		}
		if (r)
			mManager.remove(fn); 
		else 
			confirm(fn + " �Ϻ��Ǥ��ޤ���");
		return r;
	} 

	public RootMenuResource createEmptyRootMenuResource(String name) throws Exception {
		RootMenuResource r;
		if (popupResource != null) {
			if (popupResource.isAlive(name)) {
				confirm(name+" �ϡ��ɲäǤ��ޤ���");
				r = null;
			} else {
				r = new RootMenuResource();
				r.setName(name);
				popupResource.addRootMenu(r);
			}
		} else {
			r = new RootMenuResource();
			r.setName(name);
			popupResource = new PopupResource();
			popupResource.addRootMenu(r);
		}
		return r;
	}
	public void removePopup(String fn) { popupResource.remove(fn); }

	public void addRootMenu(RootMenuResource rmr) throws Exception {
		MenuResource res = getMenuResource(rmr.getMenubar_name());
		res.reArreng(rmr);
	}

	public MenuResource createMenu(String menu) throws Exception {
		MenuResource res = getMenuResource(menu);
		mManager.createMenu(res);
		return res;
	}

	public RootMenuResource getRootMenuResource(String menu, int i) throws Exception {
		MenuResource res = getMenuResource(menu);
		return res.getRootMenuResource(i);
	}	

	public ToolbarManager getToolbarManager() { return tManager; }
	public ToolbarResource createEmptyToolbarResource(String name) throws Exception {
		ToolbarResource r;

		if (tManager != null)  {
			if (tManager.isAlive(name)) {
				confirm(name+" �ϡ��ɲäǤ��ޤ���");
				r = null;
			} else {
				r = new ToolbarResource();
				r.setToolbar_name(name);
				tManager.addToolbarResource(r);
			}
		} else {
			tManager = new ToolbarManager();
			r = new ToolbarResource();
			r.setToolbar_name(name);
			tManager.addToolbarResource(r);
		}
		return r;
	}
	public void removeToolbar(String fn) { tManager.remove(fn); }
	public Vector getIconResources() { return iconResource; }
	public void addIconResource(String name, File file) throws Exception {
		int b;
		InputStream is = new FileInputStream(file);
		FileOutputStream os = new FileOutputStream(new File(getPoohResourceDir(), name));
		while ((b = is.read()) != -1) {
			os.write(b);
		}
		os.close();
		iconResource.add(name);
	}

	public void setModified(boolean b) { modified = b; }
	public boolean isModified() { return modified; }
}
